/*
	RackAFX(TM)
	Applications Programming Interface
	Derived Class Object Definition
	Copyright(c) Tritone Systems Inc. 2006-2012

	Your plug-in must implement the constructor,
	destructor and virtual Plug-In API Functions below.
*/

#pragma once

// base class
#include "plugin.h"
#include "CombFilter.h"
#include "StCombFilter.h"
#include "Delay.h"
#include "DelayAPF.h"
#include "LPFCombFilter.h"
#include "StDelayAPF.h"
#include "StLPFCombFilter.h"
#include "OnePoleLPF.h"
	
const int numCFs = 6;
const int numDAPFs = 6;
const int numLPFCFs = 6;

// abstract base class for RackAFX filters
class CReverbToolboxI : public CPlugIn
{
public:
	// RackAFX Plug-In API Member Methods:
	// The followung 5 methods must be impelemented for a meaningful Plug-In
	//
	// 1. One Time Initialization
	CReverbToolboxI();

	// 2. One Time Destruction
	virtual ~CReverbToolboxI(void);

	// 3. The Prepare For Play Function is called just before audio streams
	virtual bool __stdcall prepareForPlay();

	// 4. processAudioFrame() processes an audio input to create an audio output
	virtual bool __stdcall processAudioFrame(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels);

	// 5. userInterfaceChange() occurs when the user moves a control.
	virtual bool __stdcall userInterfaceChange(int nControlIndex);


	// OPTIONAL ADVANCED METHODS ------------------------------------------------------------------------------------------------
	// These are more advanced; see the website for more details
	//
	// 6. initialize() is called once just after creation; if you need to use Plug-In -> Host methods
	//				   such as sendUpdateGUI(), you must do them here and NOT in the constructor
	virtual bool __stdcall initialize();

	// 7. joystickControlChange() occurs when the user moves a control.
	virtual bool __stdcall joystickControlChange(float fControlA, float fControlB, float fControlC, float fControlD, float fACMix, float fBDMix);

	// 8. process buffers instead of Frames:
	// NOTE: set m_bWantBuffers = true to use this function
	virtual bool __stdcall processRackAFXAudioBuffer(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels, UINT uBufferSize);

	// 9. rocess buffers instead of Frames:
	// NOTE: set m_bWantVSTBuffers = true to use this function
	virtual bool __stdcall processVSTAudioBuffer(float** ppInputs, float** ppOutputs, UINT uNumChannels, int uNumFrames);

	// 10. MIDI Note On Event
	virtual bool __stdcall midiNoteOn(UINT uChannel, UINT uMIDINote, UINT uVelocity);

	// 11. MIDI Note Off Event
	virtual bool __stdcall midiNoteOff(UINT uChannel, UINT uMIDINote, UINT uVelocity, bool bAllNotesOff);


	// 12. MIDI Modulation Wheel uModValue = 0 -> 127
	virtual bool __stdcall midiModWheel(UINT uChannel, UINT uModValue);

	// 13. MIDI Pitch Bend
	//					nActualPitchBendValue = -8192 -> 8191, 0 is center, corresponding to the 14-bit MIDI value
	//					fNormalizedPitchBendValue = -1.0 -> +1.0, 0 is at center by using only -8191 -> +8191
	virtual bool __stdcall midiPitchBend(UINT uChannel, int nActualPitchBendValue, float fNormalizedPitchBendValue);

	// 14. MIDI Timing Clock (Sunk to BPM) function called once per clock
	virtual bool __stdcall midiClock();


	// 15. all MIDI messages -
	// NOTE: set m_bWantAllMIDIMessages true to get everything else (other than note on/off)
	virtual bool __stdcall midiMessage(unsigned char cChannel, unsigned char cStatus, unsigned char cData1, unsigned char cData2);

	// 16. initUI() is called only once from the constructor; you do not need to write or call it. Do NOT modify this function
	virtual bool __stdcall initUI();



	// Add your code here: ----------------------------------------------------------- //
	
	CStCombFilter m_CombFilters[numCFs];
	CStLPFCombFilter m_LPFCombFilters[numLPFCFs];
	CStDelayAPF m_DelayAPFs[numDAPFs];
	CDelay m_PreDlyLeft;
	CDelay m_PreDlyRight;

	void initCombFilters(int nDelayLen);
	void initLPFCombFilters(int nDelayLen);
	void initDAPFs(int nDelayLen);
	void setSampleRates();
	void resetFilters();

	void updateAllControls();

	void setCombFilterParams(int nIndex, float g, float delay);
	void setDelayAPFParams(int nIndex, float g, float delay);
	void setLPFCombFilterParams(int nIndex, float g, float delay, float g2);

	// models
	void doPreDelayAndERs(float xn, float& yn, float& erMix, bool bLeftChannel);
	void doSchroeder(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels);
	void doMoorer(float* pInputBuffer, float* pOutputBuffer, UINT uNumInputChannels, UINT uNumOutputChannels);

	// END OF USER CODE -------------------------------------------------------------- //


	// ADDED BY RACKAFX -- DO NOT EDIT THIS CODE!!! ----------------------------------- //
	//  **--0x07FD--**

	float m_fMix;
	float m_fPDelay;
	float m_fRT60;
	float m_f_LPFCF_g;
	float m_fCF1Dly;
	float m_fCF2Dly;
	float m_fCF3Dly;
	float m_fCF4Dly;
	float m_fCF5Dly;
	float m_fCF6Dly;
	float m_fERMix;
	float m_fLPCF1Dly;
	float m_fLPCF2Dly;
	float m_fLPCF3Dly;
	float m_fLPCF4Dly;
	float m_fLPCF5Dly;
	float m_fLPCF6Dly;
	float m_fAPF1Dly;
	float m_fAPF2Dly;
	float m_fAPF3Dly;
	float m_fAPF4Dly;
	float m_fAPF5Dly;
	float m_fAPF6Dly;
	float m_fAPF1_g;
	float m_fAPF2_g;
	float m_fAPF3_g;
	float m_fAPF4_g;
	float m_fAPF5_g;
	float m_fAPF6_g;
	UINT m_uReverbModel;
	enum{Schroeder,Moorer,Moorer_mk2};
	UINT m_uInvertOdd;
	enum{SWITCH_OFF,SWITCH_ON};
	UINT m_uMonoMode;
	UINT m_uER1;
	UINT m_uER2;
	UINT m_uERMode;
	enum{MIX,INSERT,BOTH};

	// **--0x1A7F--**
	// ------------------------------------------------------------------------------- //

};

































































